unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus;

type
  TfMain = class(TForm)
    L1: TLabel;
    L2: TLabel;
    stLives: TStaticText;
    stMoney: TStaticText;
    MainMenu: TMainMenu;
    mFile: TMenuItem;
    mAbout: TMenuItem;
    smRestart: TMenuItem;
    smSave: TMenuItem;
    smLoad: TMenuItem;
    smQuit: TMenuItem;
    fdOpen: TOpenDialog;
    fdSave: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormPaint(Sender: TObject);
    procedure GoUp;
    procedure GoDown;
    procedure GoLeft;
    procedure GoRight;
    procedure smQuitClick(Sender: TObject);
    procedure smSaveClick(Sender: TObject);
    procedure smLoadClick(Sender: TObject);
    procedure mAboutClick(Sender: TObject);
    procedure smRestartClick(Sender: TObject);
    procedure HandleMessages(var Msg: tMsg; var Handled: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  MapTop=10;
  MapLeft=10;
  MapLastX=29;
  MapLastY=29;
  CellHeight=16;
  CellWidth=16;
  CellTypesNum=7;

  cFree = 1;
  cBlock = 2;
  cMoney = 3;
  cHealth = 4;
  cBomb = 5;
  cMegaBomb = 6;
  cTelePort = 7;

  CellsNameList: array[1..CellTypesNum] of string =
    (
      'free.bmp',
      'block.bmp',
      'money.bmp',
      'health.bmp',
      'bomb.bmp',
      'megabomb.bmp',
      'teleport.bmp'
    );

  BlockNum = 100;
  MoneyNum = 10;
  HealthNum = 10;
  BombNum = 25;
  MegaBombNum = 1;
  TelePortNum = 5;

type
  TMan = record
           x,y: integer;
           lives: integer;
           money: integer;
         end;

var
  fMain: TfMain;

  imgDark,imgMan: TBitMap;
  imgCells: array[1..CellTypesNum] of TBitMap;

  Map,Fog: array[0..MapLastX] of array[0..MapLastY] of integer;

  Man: TMan;

  TargetX,TargetY: integer;

  SaveFile: file;

  Path: string;

  WatchKeys: boolean;

implementation

{$R *.DFM}

procedure TfMain.HandleMessages(var Msg: tMsg; var Handled: Boolean);
begin
  if not WatchKeys then Exit;

  if (Msg.Message = WM_KeyDown) and
    (Msg.wParam in [VK_UP, VK_DOWN, VK_LEFT, VK_RIGHT]) then
  begin
    case Msg.wParam of
      VK_UP: GoUp;
      VK_DOWN: GoDown;
      VK_LEFT: GoLeft;
      VK_RIGHT: GoRight;
    end;
    Handled := True;
  end;
end;

procedure TfMain.FormCreate(Sender: TObject);
begin
  Application.OnMessage := HandleMessages;
end;


procedure DrawCell(x,y: integer; iCell: integer);
var xx,yy: integer;
begin
  xx := MapLeft + x * CellWidth;
  yy := MapTop + y * CellHeight;

  case iCell of
       0: fMain.Canvas.Draw(xx,yy,imgMan);
       1..CellTypesNum:
            begin
              If Fog[x][y] = 1
                 then
                   fMain.Canvas.Draw(xx,yy,imgCells[iCell])
                 else
                   fMain.Canvas.Draw(xx,yy,imgDark)
            end;
  end;
end;

procedure DrawMap;
var x,y: integer;
begin
 for y:=0 to MapLastY do
 for x:=0 to MapLastX do
     DrawCell(x,y,Map[x][y]);
end;

procedure OpenArea(x,y: integer);
var ix,iy: integer;
begin
  for iy:=y-1 to y+1 do
  for ix:=x-1 to x+1 do
      if (ix >= 0) and (iy >= 0)
         and (ix <= MapLastX) and (iy <= MapLastY)
             then
               begin
                 Fog[ix][iy] := 1;
                 DrawCell(ix,iy,Map[ix][iy]);
               end;
end;

procedure ShowMan(yes: boolean);
begin
  OpenArea(Man.x,Man.y);

  if yes then DrawCell(Man.x,Man.y,0)
         else DrawCell(Man.x,Man.y,1);
end;

procedure ReportMan;
begin
  fMain.stLives.Caption := IntToStr(Man.Lives);
  fMain.stMoney.Caption := IntToStr(Man.Money);
end;

procedure MoveManTo(x,y: integer);
begin
  ShowMan(false);
  Man.x := x;
  Man.y := y;
  ShowMan(true);
  Map[x][y] := cFree;
end;

procedure ThrowMan;
var x,y: integer;
begin
  ShowMan(false);

  Repeat
    x := Random(MapLastX);
    y := Random(MapLastY);
  Until Map[x][y] = cFree;

  Man.x := x;
  Man.y := y;

  ShowMan(true);
end;

function WhatIs(x,y: integer): integer;
begin
  If (x < 0) or (y < 0) or (x > MapLastX) or (y > MapLastY)
     then
       WhatIs := cBlock  // not available
     else
       WhatIs := Map[x][y];
end;

procedure DriveMan;
var Target: integer;
begin
  Target := WhatIs(TargetX,TargetY);

  Case Target of
    cFree:  MoveManTo(TargetX,TargetY);
    cMoney: begin
              Man.Money := Man.Money + 50;
              MoveManTo(TargetX,TargetY);
              ReportMan;
              
              If Man.Money = (MoneyNum * 50)
                 then
                   begin
                     WatchKeys := false;
                     ShowMessage('Rulezzz... You have got `em all!');
                     WatchKeys := true;
                     Application.Terminate;
                   end;
            end;
    cHealth: begin
              Man.lives := Man.lives + 1;
              MoveManTo(TargetX,TargetY);
              ReportMan;
            end;
    cBomb: begin
              Man.lives := Man.lives - 1;
              If Man.Lives > 0 then
                 begin
                  WatchKeys := false;
                  ShowMessage('You were hit by hidden bomb!');
                  MoveManTo(TargetX,TargetY);
                  ReportMan;
                  WatchKeys := true;
                 end
              else
                 begin
                  WatchKeys := false;
                  ShowMessage('Game over; you were killed by hidden bomb!');
                  WatchKeys := true;
                  Application.Terminate;
                 end;
            end;
    cMegaBomb: begin
              WatchKeys := false;
              ShowMessage('Game over; you were smashed by MEGABOMB!');
              WatchKeys := true;
              Application.Terminate;
            end;
    cTelePort: begin
              WatchKeys := false;
              ShowMessage('Ooops! You were thrown away by evil force!');
              ThrowMan;
              WatchKeys := true;
            end;
    else;
  end;
end;

procedure CleanMap;
var x,y: integer;
begin
 for y:=0 to MapLastY do
 for x:=0 to MapLastX do
     begin
       Map[x][y] := cFree;       // all map is free...
       Fog[x][y] := 0;           // ...and dark.
     end;
end;

procedure FillMapWith(WithWhat: integer; HowMany: integer);
var x,y: integer;
    i: integer;
begin
  for i:=1 to HowMany do
      begin
        Repeat
          x := Random(MapLastX);
          y := Random(MapLastY);
        until Map[x][y] = cFree;

        Map[x][y] := WithWhat;
      end;
end;

procedure RestartGame;
begin
  randomize;

  CleanMap;

  FillMapWith(cBlock,BlockNum);
  FillMapWith(cMoney,MoneyNum);
  FillMapWith(cHealth,HealthNum);
  FillMapWith(cBomb,BombNum);
  FillMapWith(cMegaBomb,MegaBombNum);
  FillMapWith(cTelePort,TelePortNum);

  With Man do
    begin
      Repeat
        x := random(MapLastX);
        y := random(MapLastY);
      until Map[x][y] = cFree;

      lives := 5;
      money := 0;
    end;

  WatchKeys := true;
end;

procedure TfMain.FormShow(Sender: TObject);
var i: integer;
begin
  fdOpen.InitialDir := Path;
  fdSave.InitialDir := Path;

  imgDark := TBitmap.Create;
  imgDark.LoadFromFile(Path+'dark.bmp');
  imgMan := TBitmap.Create;
  imgMan.LoadFromFile(Path+'man.bmp');

  for i:=1 to CellTypesNum do
    begin
      imgCells[i] := TBitmap.Create;
      imgCells[i].LoadFromFile(Path+CellsNameList[i]);
    end;

  RestartGame;
end;

procedure TfMain.FormClose(Sender: TObject; var Action: TCloseAction);
var i: integer;
begin
  imgDark.Free;
  imgMan.Free;

  for i:=1 to CellTypesNum do imgCells[i].Free;
end;

procedure TfMain.FormPaint(Sender: TObject);
begin
  DrawMap;
  ShowMan(true);
  ReportMan;
end;

procedure TfMain.GoUp;
begin
  TargetX := Man.x;
  TargetY := Man.y - 1;
  DriveMan;
end;

procedure TfMain.GoDown;
begin
  TargetX := Man.x;
  TargetY := Man.y + 1;
  DriveMan;
end;

procedure TfMain.GoLeft;
begin
  TargetX := Man.x - 1;
  TargetY := Man.y;
  DriveMan;
end;

procedure TfMain.GoRight;
begin
  TargetX := Man.x + 1;
  TargetY := Man.y;
  DriveMan;
end;

procedure TfMain.smQuitClick(Sender: TObject);
begin
  WatchKeys := false;

  If MessageDlg('Do you really wish this?!?!', mtConfirmation,
     [mbYes, mbNo], 0) = id_Yes
     then Close;

  WatchKeys := true;
end;

procedure TfMain.smSaveClick(Sender: TObject);
begin
  If fdSave.Execute then
     begin
       AssignFile(SaveFile,fdSave.FileName);
       Rewrite(SaveFile,1);

       BlockWrite(SaveFile,Map,SizeOf(Map));
       BlockWrite(SaveFile,Fog,SizeOf(Fog));
       BlockWrite(SaveFile,Man,SizeOf(Man));

       CloseFile(SaveFile);
     end;
end;

procedure TfMain.smLoadClick(Sender: TObject);
begin
  If fdOpen.Execute then
     begin
       AssignFile(SaveFile,fdOpen.FileName);
       Reset(SaveFile,1);

       BlockRead(SaveFile,Map,SizeOf(Map));
       BlockRead(SaveFile,Fog,SizeOf(Fog));
       BlockRead(SaveFile,Man,SizeOf(Man));

       CloseFile(SaveFile);
     end;

  Refresh;   
end;

procedure TfMain.mAboutClick(Sender: TObject);
begin
 WatchKeys := false;
 ShowMessage('The Hobo in the Dark. (C) 2003.');
 WatchKeys := true;
end;

procedure TfMain.smRestartClick(Sender: TObject);
begin
  RestartGame;
  Refresh;
end;

end.

